local L = AceLibrary("AceLocale-2.2"):new("Capping")

function Capping:LoadAVData()
	self.avdata = {
		["graveyard"]	= "Interface\\Icons\\Spell_Shadow_AnimateDead",
		["tower"] 	= "Interface\\Icons\\Ability_Hunter_Quickshot",
		["tree"]		= "Interface\\Icons\\Spell_Nature_NaturesBlessing",
		["ice"]		= "Interface\\Icons\\Spell_Frost_Glacier",
	}
end

function Capping:StartAV()
	if not self.avdata then
		self:LoadAVData()
	end
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "AVAssaults")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL", "AVStartTimer")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", "SnowfallAlliance") 
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", "SnowfallHorde") 

	self:RegisterEvent("GOSSIP_SHOW", "AVQuests")
	self:RegisterEvent("QUEST_PROGRESS", "AVQuests")
	self:RegisterEvent("QUEST_COMPLETE", "AVQuests")
end

function Capping:AVStartTimer()
	self:CheckStartTimer()
end

function Capping:AVAssaults()
	if string.find(arg1, L["avunderattack"]) then
		local _,_,node,faction = string.find(arg1, L["avunderattack"])
		node = string.gsub(node, L["The "], "")
		if faction == L["Horde"] then 
			self:StopBar(node)
			self:StartBar(node, 303, self.avdata[self:NodeType(node)], "Red")
		else
			self:StopBar(node)
			self:StartBar(node, 303, self.avdata[self:NodeType(node)], "Blue")
		end
	elseif string.find(arg1, L["avtaken"]) then
		local _,_,node = string.find(arg1, L["avtaken"])
		node = string.gsub(node, L["The "], "")
		self:StopBar(node)
	elseif string.find(arg1, L["avdestroyed"]) then
		local _,_,node = string.find(arg1, L["avdestroyed"])
		node = string.gsub(node, L["The "], "")
		self:StopBar(node)
	elseif string.find(arg1, L["Wicked, wicked, mortals!"]) then 
		self:StartBar(L["Ivus begins moving"], 603, self.avdata.tree, "Blue")
	elseif string.find(arg1, L["WHO DARES SUMMON LOKHOLAR"]) then
		self:StartBar(L["Lokholar begins moving"], 603, self.avdata.ice, "Red")
	end
end			

function Capping:SnowfallHorde()
	--Initial capture of Snowfall for horde
	if string.find(strlower(arg1), strlower(L["Snowfall Graveyard"])) then
		self:StopBar(L["Snowfall Graveyard"])
		self:StartBar(L["Snowfall Graveyard"], 303, self.avdata.graveyard, "Red")
	end
end

function Capping:SnowfallAlliance()
	--Initial capture of Snowfall for alliance
	if string.find(strlower(arg1), strlower(L["Snowfall Graveyard"])) then
		self:StopBar(L["Snowfall Graveyard"])
		self:StartBar(L["Snowfall Graveyard"], 303, self.avdata.graveyard, "Blue")
	end
end

-- determines if a node's a graveyard or tower
function Capping:NodeType(node)
	-- check for towers, if not...assume it's a graveyard
	if string.find(node, L["Bunker"]) or string.find(node, L["Tower"]) then
		return "tower"
	end
	return "graveyard"
end

function Capping:AVQuests()
	if not self.db.profile.avquest then
		return
	end
	local target = UnitName("target")
	if event == "GOSSIP_SHOW" or event=="QUEST_PROGRESS" then
		if target == L["Smith Regzar"] or target == L["Murgot Deepforge"] then
			-- Open Quest to Smith or Murgot
			if GetGossipOptions() and string.find(GetGossipOptions(), L["Upgrade to"] ) then
				SelectGossipOption(1)
			elseif (self:GetItemCount(L["Armor Scraps"]) >= 20) then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Primalist Thurloga"] then
			local num = self:GetItemCount(L["Stormpike Soldier's Blood"])
			if num >= 5 then
				SelectGossipAvailableQuest(2)
			elseif num > 0 then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Arch Druid Renferal"] then
			local num = self:GetItemCount(L["Storm Crystal"])
			if num >= 5 then
				SelectGossipAvailableQuest(2)
			elseif num > 0 then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Stormpike Ram Rider Commander"] then
			if self:GetItemCount(L["Frostwolf Hide"]) > 0 then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Frostwolf Wolf Rider Commander"] then
			if self:GetItemCount(L["Alterac Ram Hide"]) > 0 then
				SelectGossipAvailableQuest(1)
			end
		end
	end
	if event == "QUEST_PROGRESS" then
		if (target == L["Wing Commander Jeztor"] and self:GetItemCount(L["Stormpike Lieutenant's Flesh"]) == 0) or
		   (target == L["Wing Commander Guse"] and self:GetItemCount(L["Stormpike Soldier's Flesh"]) == 0) or
		   (target == L["Wing Commander Mulverick"] and self:GetItemCount(L["Stormpike Commander's Flesh"]) == 0) or
		   (target == L["Wing Commander Vipore"] and self:GetItemCount(L["Frostwolf Lieutenant's Medal"]) == 0) or
		   (target == L["Wing Commander Slidore"] and self:GetItemCount(L["Frostwolf Soldier's Medal"]) == 0) or
		   (target == L["Wing Commander Ichman"] and self:GetItemCount(L["Frostwolf Commander's Medal"]) == 0) then
			return
	   	end
		CompleteQuest()
	elseif event == "QUEST_COMPLETE" then
		GetQuestReward(0)
	end
end

function Capping:GetItemCount(ItemName)
	local num, bagNr, bagSlot = 0
	for bagNr = 0,4,1 do
		for bagSlot = 1,GetContainerNumSlots(bagNr),1 do
			if self:GetItemName(bagNr, bagSlot) == ItemName then
				local _,itemCount = GetContainerItemInfo(bagNr, bagSlot)
				num = num + itemCount
			end
		end
	end
	return num
end 

function Capping:GetItemName(bag, slot)
	local link
	if bag == -1 then	
		link = GetInventoryItemLink("player", slot)
	else 
		link = GetContainerItemLink(bag, slot)
	end
	if link then
		local _,_,name = string.find(link, "^.*%[(.*)%].*$")
		return name
	else
		return ""
	end
end


