local L = AceLibrary("AceLocale-2.2"):new("Capping")

Capping = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "CandyBar-2.0")

function Capping:OnInitialize()
	self:Hook("WorldStateFrame_ToggleMinimap", true)
	self:SecureHook("WorldStateScoreFrame_Update")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self:RegisterDB("CappingDB")
	self:RegisterDefaults("profile", {
		av = true,
		avquest = true,
		ab = true,
		wsg = true,
		arena = true,
		queue = true,
		autoshow = true,
		mapscale = 1.3,
		narrow = true,
		texture = "BantoBarReverse",
		fontsize = 10,
		width = 200,
		height = 16,
		scale = 1,
		disableworld = true,
		classcolor = true,
	} )
	
	self:RegisterChatCommand({ "/capping"}, {
		type = "execute", name = "Capping", desc = "Capping",
		func = function() self:Options() end,
	})
	
	if SupplyAndDemand and IsAddOnLoadOnDemand("Capping") then
		self:PLAYER_ENTERING_WORLD()
	end
	
	self.surface = AceLibrary("Surface-1.0")
	self.surface:Register("BantoBarReverse", "Interface\\AddOns\\Capping\\BantoBarReverse")
	self.surface:Register("NotSteel", "Interface\\AddOns\\Capping\\NotSteel")
	self.surface:Register("smooth", "Interface\\AddOns\\Capping\\smooth")

end

function Capping:PLAYER_ENTERING_WORLD()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "CheckQueueTimer")
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:ScheduleEvent(self.ZoneCheck, 1, self)
end

function Capping:Options()
	local self = Capping
	if not self.bulkdone then
		self:BulkWork()
	end
	if not self.opts then
		self:LoadOptions()
	end
	self.dewdrop:Open( CappingFrame,
		'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		'cursorX', true, 'cursorY', true )
end

function Capping:BulkWork()
	self.paint = AceLibrary("PaintChips-2.0")
	
	-- handles battleground queue timers
	self.currentq = { }
	
	self.activebars = { }
	self.paint:RegisterColor("grey", 0.3, 0.3, 0.3)
	self.paint:RegisterColor("lightgrey", 0.6, 0.6, 0.6)
	
	self:CreateBackbone()

	self.bulkdone = true
end

function Capping:LoadOptions()
	self.dewdrop = AceLibrary("Dewdrop-2.0")

	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "Capping",
				order = 1,
			},
			av = {
				type = "group",
				name = L["Alterac Valley"],
				desc = L["Alterac Valley"],
				order = 2,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.av = v self:ToggleMod(v, L["Alterac Valley"]) end,
						get = function() return self.db.profile.av end,
					},
					quests = {
						type = "toggle",
						name = L["Auto quest turnins"],
						desc = L["Enable Alterac Valley automatic quest turnins"],
						order = 2,
						set = function(v) self.db.profile.avquest = v end,
						get = function() return self.db.profile.avquest end,
					},
				},
			},
			ab = {
				type = "group",
				name = L["Arathi Basin"],
				desc = L["Arathi Basin"],
				order = 3,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.ab = v self:ToggleMod(v, L["Arathi Basin"]) end,
						get = function() return self.db.profile.ab end,
					},
				},
			},
			wsg = {
				type = "group",
				name = L["Warsong Gulch"],
				desc = L["Warsong Gulch"],
				order = 4,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.wsg = v self:ToggleMod(v, L["Warsong Gulch"]) end,
						get = function() return self.db.profile.wsg end,
					},
				},
			},
			bars = {
				type = "group",
				name = L["Bar"],
				desc = L["Statusbar options"],
				order = 5,
				args = {
					scale = {
						type = "range",
						name = L["Scale"],
						desc = L["Change statusbar scale"],
						set = function(v) self.db.profile.scale = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.scale end,
						min = 0.5, max = 2, step = 0.1,
						order = 1,
					},
					width = {
						type = "range",
						name = L["Width"],
						desc = L["Change statusbar width"],
						set = function(v) self.db.profile.width = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.width end,
						min = 50, max = 300, step = 10,
						order = 2,
					},
					height = {
						type = "range",
						name = L["Height"],
						desc = L["Change statusbar thickness"],
						set = function(v) self.db.profile.height = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.height end,
						min = 10, max = 40, step = 2,
						order = 3,
					},
					texture = {
						type = "text",
						name = L["Texture"],
						desc = L["Statusbar textures"],
						set = function(v) self.db.profile.texture = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.texture end,
						validate = self.surface:List(),
						order = 4,
					},
					fontsize = {
						type = "range",
						name = L["Font size"],
						desc = L["Change statusbar font size"],
						set = function(v) self.db.profile.fontsize = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.fontsize end,
						min = 8, max = 20, step = 0.2,
						order = 5,
					},
					test = {
						type = "execute",
						name = L["Test"],
						desc = L["Start a test bar"],
						func = function() self:StartBar("Testing", 30, "Interface\\Icons\\Ability_ThunderBolt", "lightgrey") end,
						order = 6,
					},
				},
			},
			other = {
				type = "group",
				name = L["Other"],
				desc = L["Other options"],
				order = 6,
				args = {
					autoshowmap = {
						type = "toggle",
						name = L["Auto show map"],
						desc = L["Auto show the battlefield minimap on entry"],
						set = function(v) self.db.profile.autoshow = v end,
						get = function() return self.db.profile.autoshow end,
						order = 1, 
					},
					narrow = {
						type = "toggle",
						name = L["Narrow map mode"],
						desc = L["Narrow the battlefield minimap, removing some empty space"],
						set = function(v) self.db.profile.narrow = v self:SetNarrowMap() end,
						get = function() return self.db.profile.narrow end,
						order = 2,
					},
					world = {
						type = "toggle",
						name = L["Disable World PVP Map"],
						desc = L["Disable the battlefield minimap from showing outside battlegrounds"],
						set = function(v) self.db.profile.disableworld = v end,
						get = function() return self.db.profile.disableworld end,
						order = 3,
					},
					mapscale = {
						type = "range",
						name = L["Map scale"],
						desc = L["Change the default scale of the battlefield minimap"],
						set = function(v) 
							self.db.profile.mapscale = v 
							if BattlefieldMinimap and BattlefieldMinimap:IsVisible() then 
								BattlefieldMinimap:SetScale(v) 
							end
						end,
						get = function() return self.db.profile.mapscale end,
						min = 0.5, max = 2.0, step = 0.1,
						order = 4,
					},
					queue = {
						type = "toggle",
						name = L["Queue Timer"],
						desc = L["Enable statusbar for queues"],
						set = function(v) self.db.profile.queue = v end,
						get = function() return self.db.profile.queue end,
						order = 5,
					},
					scorecolor = {
						type = "toggle",
						name = L["Toggle class color"],
						desc = L["Enable/disable class color indicators on the scoreboard"],
						set = function(v) self.db.profile.classcolor = v   end,
						get = function() return self.db.profile.classcolor end,
						order = 6,
					},
				},
			},
			anchor = {
				type = "execute",
				name = L["Show/Hide Anchor"],
				desc = L["Show/Hide the bars anchor (can also left-click a statusbar)"],
				func = function() 
					if CappingFrame:IsVisible() then 
						CappingFrame:Hide()
					else 
						CappingFrame:Show()
					end 
				end,
				order = 7,
			},
		},
	}
	
	self:RegisterEvent("Surface_Registered")
end

function Capping:Surface_Registered()
	self.opts.args.bar.args.texture.validate = self.surface:List()
end

-- zone change
function Capping:ZONE_CHANGED_NEW_AREA()
	self:ResetAll()
	self:ZoneCheck()
end

-- reset all timers and unregister events
function Capping:ResetAll()
	self:UnregisterAllEvents()
	self:CancelAllScheduledEvents()
	if self.activebars then
		for i=1,getn(self.activebars),1 do
			local value = self.activebars[i]
			self:StopBar(value)
		end
		self.activebars = { }
	end
	if TargetAlliance then
		TargetAlliance:Hide()
		TargetHorde:Hide()
	end
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "CheckQueueTimer")
end

-- check entry to a battleground zone
function Capping:ZoneCheck()
	local self = Capping
	local currentZone = GetZoneText()
	if currentZone == L["Warsong Gulch"] and self.db.profile.wsg then
		if not self.bulkdone then self:BulkWork() end
		self:StartWSG()
		self:ShowBGMap()
	elseif currentZone == L["Arathi Basin"] and self.db.profile.ab then
		if not self.bulkdone then self:BulkWork() end
		self:StartAB()
		self:ShowBGMap()
	elseif currentZone == L["Alterac Valley"] and self.db.profile.av then
		if not self.bulkdone then self:BulkWork() end
		self:StartAV()
		self:ShowBGMap()
	elseif strfind(currentZone, L["Arena"]) and self.db.profile.arena then
		if not self.bulkdone then self:BulkWork() end
		self:StartArena()
		-- there is no minimap for arenas
	elseif BattlefieldMinimap then
		BattlefieldMinimap:Hide()
	end
end

-- show the battlefield minimap
function Capping:ShowBGMap()
	if not BattlefieldMinimap then LoadAddOn("Blizzard_BattlefieldMinimap") end
	BattlefieldMinimap:SetScale(self.db.profile.mapscale)
	self:SetNarrowMap()
	if self.db.profile.autoshow then BattlefieldMinimap:Show() end
end

-- prevents the bgmap from showing outside of battlegrounds
function Capping:WorldStateFrame_ToggleMinimap()
	local zone = GetZoneText()
	if not self.db.profile.disableworld or zone == L["Alterac Valley"] or zone == L["Arathi Basin"] or zone == L["Warsong Gulch"] then
		self.hooks.WorldStateFrame_ToggleMinimap()
	end
end

-- narrow the default battlefield minimap to remove unnecessary areas taking space
function Capping:SetNarrowMap(disable)
	if self.db.profile.narrow and not self.narrowed and not disable then
		self.narrowed = 1
		BattlefieldMinimap1:Hide()
		BattlefieldMinimap4:Hide()
		BattlefieldMinimap5:Hide()
		BattlefieldMinimap8:Hide()
		BattlefieldMinimap9:Hide()
		BattlefieldMinimap12:Hide()
		BattlefieldMinimapBackground:SetWidth(256 / 2)
		BattlefieldMinimapBackground:SetPoint("TOPLEFT", -12 + 64, 12)
		BattlefieldMinimapCorner:SetPoint("TOPRIGHT", -2 - 64 + 12, 3 + 1)
		BattlefieldMinimapCorner:SetWidth(32 * 0.75)
		BattlefieldMinimapCorner:SetHeight(32 * 0.75)
		BattlefieldMinimapCloseButton:SetPoint("TOPRIGHT", 2 - 64 + 12 - 1, 7)
		BattlefieldMinimapCloseButton:SetWidth(32 * 0.75)
		BattlefieldMinimapCloseButton:SetHeight(32 * 0.75)
		BattlefieldMinimap:SetPoint("TOPLEFT", BattlefieldMinimapTab, "BOTTOMLEFT", 0 - 64, -5)
		self:Hook("MouseIsOver", function(frame, top, bottom, left, right)
			if frame == BattlefieldMinimap then 
				left = left + 64
				right = right - 64
			end
			return self.hooks.MouseIsOver(frame, top, bottom, left, right)
		end, true)
	elseif disable or (not self.db.profile.narrow and self.narrowed) then
		-- setting things back to blizz's default
		self.narrowed = nil
		BattlefieldMinimap1:Show()
		BattlefieldMinimap4:Show()
		BattlefieldMinimap5:Show()
		BattlefieldMinimap8:Show()
		BattlefieldMinimap9:Show()
		BattlefieldMinimap12:Show()
		BattlefieldMinimapBackground:SetWidth(256)
		BattlefieldMinimapBackground:SetPoint("TOPLEFT", -12, 12)
		BattlefieldMinimapCorner:SetPoint("TOPRIGHT", -2, 3)
		BattlefieldMinimapCorner:SetWidth(32)
		BattlefieldMinimapCorner:SetHeight(32)
		BattlefieldMinimapCloseButton:SetPoint("TOPRIGHT", 2, 7)
		BattlefieldMinimapCloseButton:SetWidth(32)
		BattlefieldMinimapCloseButton:SetHeight(32)
		BattlefieldMinimap:SetPoint("TOPLEFT", BattlefieldMinimapTab, "BOTTOMLEFT", 0, -5)
		self:Unhook("MouseIsOver")
	end
end

-- scoreboard class color indicators - @ name
function Capping:WorldStateScoreFrame_Update()
	if not self.db.profile.classcolor then return end
	for i = 1, MAX_WORLDSTATE_SCORE_BUTTONS do
		local index = FauxScrollFrame_GetOffset(WorldStateScoreScrollFrame) + i
		if index > GetNumBattlefieldScores() then break end
		local _,_,_,_,_,_,_,_,class = GetBattlefieldScore(index)
		local nametext = getglobal("WorldStateScoreButton" .. i .. "NameButtonName"):GetText()
		getglobal("WorldStateScoreButton" .. i .. "NameButtonName"):SetText(" |cFF"..self.paint:GetHex(strupper(class)).."@|r "..nametext)
	end
end

-- the 2 min timer to port to a certain battleground
function Capping:CheckQueueTimer()
	if not self.db.profile.queue then return end
	if not self.bulkdone then self:BulkWork() end
	
	for map in pairs(self.currentq) do
		self.currentq[map] = "active"
	end
	for i=1,MAX_BATTLEFIELD_QUEUES do
		local status, map, _ = GetBattlefieldStatus(i)
		if status == "confirm" and not self.currentq[map] then
			self.currentq[map] = status
			if strfind(map, L["Arena"]) then
				self:StartBar(L["Port: "]..map, 60, "Interface\\Icons\\Ability_TownWatch", "lightgrey", true, true)
			else
				self:StartBar(L["Port: "]..map, 120, "Interface\\Icons\\Ability_TownWatch", "lightgrey", true, true)
			end
		elseif status == "confirm" then
			self.currentq[map] = status
		elseif self.currentq[map] then
			self.currentq[map] = nil
			self:StopBar(L["Port: "]..map)
		end
	end
	for map in pairs(self.currentq) do
		if self.currentq[map] == "active" then
			self.currentq[map] = nil
			self:StopBar(L["Port: "]..map)
		end
	end
end

-- timer for when a battleground begins
function Capping:CheckStartTimer()
	if string.find(arg1, L["begins in 1 minute"]) or string.find(arg1, L["begin in 1 minute"]) or string.find(arg1, L["1 minute until"]) or string.find(arg1, L["One minute until"]) then
		self:StartStartTimer(63)
	elseif string.find(arg1, L["begins in 30 seconds"]) or string.find(arg1, L["begin in 30 seconds"]) or string.find(arg1, L["30 seconds until"]) or string.find(arg1, L["Thirty seconds until"]) then
		self:StartStartTimer(63, 32)
	elseif string.find(arg1, L["Fifteen seconds until"]) then
		self:StartStartTimer(63, 16)
	end
end

function Capping:StartStartTimer(total, remain)
	if not self:IsCandyBarRegistered(L["Battleground Begins"]) then
		self:StartBar(L["Battleground Begins"], total, "Interface\\Icons\\Spell_Holy_PrayerOfHealing", "lightgrey")
	end
	if remain then self:SetCandyBarTimeLeft(L["Battleground Begins"], remain) end
end

-- creates the statusbar anchor and registers candybar group
function Capping:CreateBackbone()
	self.mainframe = CreateFrame("Button", "CappingFrame", UIParent)
	self.mainframe:SetHeight(16)
	self.mainframe:SetWidth(200)
	self.mainframe:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	                             tile = true, tileSize = 16,
	                             insets = { left = 0, right = 0, top = 0, bottom = 0 }
	                          })
	self.mainframe:SetBackdropColor(0.3, 0.3, 0.3, 0.5)
	self.mainframe:SetMovable(true)
	self.mainframe:EnableMouse(true)
	self.mainframe:RegisterForDrag("LeftButton")
	self.mainframe:RegisterForClicks("RightButtonUp")
	self.mainframe:SetScript("OnDragStart", function() this:StartMoving() end)
	self.mainframe:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing()
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			self.db.profile.x = floor(d + 0.5)
			self.db.profile.y = floor(e + 0.5)
		end
	end)
	self.mainframe:SetScript("OnClick", function()
		if arg1 == "RightButton" then
			Capping:Options()
		end
	end)
	
	self.mainframe:Hide()
	
	self.mainframe.text = self.mainframe:CreateFontString("CappingFrameText", "OVERLAY")
	self.mainframe.text:SetFont(GameFontHighlightSmall:GetFont())
	self.mainframe.text:SetText("Capping")
	self.mainframe.text:ClearAllPoints()
	self.mainframe.text:SetAllPoints(self.mainframe)
	
	if not self.db.profile.x then
		self.mainframe:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
	else
		self.mainframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	
	self:RegisterCandyBarGroup("CappingBottom")
	self:SetCandyBarGroupPoint("CappingBottom", "TOP", self.mainframe, "TOP", 0, -15)
	
	self:RegisterCandyBarGroup("CappingTop")
	self:SetCandyBarGroupPoint("CappingTop", "BOTTOM", self.mainframe, "BOTTOM", 0, 15)
	self:SetCandyBarGroupGrowth("CappingTop", true)
end

-- begin a statusbar timer
-- - name: id of bar and text of bar
-- - duration: full duration of bar
-- - icon: icon location
-- - color: paintchips color
-- - nonactive: set true if you want to continue this bar timer after zoning
-- - separate: set true to anchor to the top instead of bottom
-- - specialText: bar text if different from name
function Capping:StartBar(name, duration, icon, color, nonactive, separate, specialText)
	if not specialText then
		self:RegisterCandyBar(name, duration, name, icon, color)
	else
		self:RegisterCandyBar(name, duration, specialText, icon, color)
	end
	self:SetCandyBarTexture(name, self.surface:Fetch(self.db.profile.texture))
	self:SetCandyBarWidth(name, self.db.profile.width)
	self:SetCandyBarHeight(name, self.db.profile.height)
	self:SetCandyBarFontSize(name, self.db.profile.fontsize)
	self:SetCandyBarScale(name, self.db.profile.scale)
	self:SetCandyBarFade(name, 1)
	self:SetCandyBarBackgroundColor(name, "Grey", 0.6)
	if separate then
		self:RegisterCandyBarWithGroup(name, "CappingTop")
	else
		self:RegisterCandyBarWithGroup(name, "CappingBottom")
	end
	self:SetCandyBarOnClick(name, function(name, button) self:OnClick(name, button) end)

	self:StartCandyBar(name, true)
	if not nonactive then
		tinsert(self.activebars, name)
	end
end

-- simply cancels a timer
function Capping:StopBar(name)
	self:StopCandyBar(name)
	self:UnregisterCandyBar(name)
end

-- updates active bars with new textures if needed
function Capping:UpdateActiveBars()
	for i=1,getn(self.activebars),1 do
		local value = self.activebars[i]
		self:SetCandyBarTexture(value, self.surface:Fetch(self.db.profile.texture))
		self:SetCandyBarWidth(value, self.db.profile.width)
		self:SetCandyBarHeight(value, self.db.profile.height)
		self:SetCandyBarFontSize(value, self.db.profile.fontsize)
		self:SetCandyBarScale(value, self.db.profile.scale)
		self:UpdateCandyBarGroup("CappingBottom")
		self:UpdateCandyBarGroup("CappingTop")
	end
end

-- in case a user wants to enable/disable the current bg's mod
function Capping:ToggleMod(enable, zone)
	if GetZoneText() == zone then
		if not enable then
			self:ResetAll()
		else
			self:ZoneCheck()
		end
	end
end

-- gives some user friendliness to the Candy Bars
-- - left-click to toggle anchor
-- - shift left-click to send maybe useful info to /bg
-- - ctrl left-click to send to /s
-- - right-click to bring up config menu
-- - shift right-click to cancel timer
function Capping:OnClick(name, button)
	if button == "LeftButton" then
		if IsShiftKeyDown() then
			local _,time,elapsed,_ = self:CandyBarStatus(name)
			SendChatMessage(string.format(L["%s - %s remaining"], name, self:FormatChatTime(time-elapsed)), "SAY")
		elseif IsControlKeyDown() then
			local _,time,elapsed,_ = self:CandyBarStatus(name)
			SendChatMessage(string.format(L["%s - %s remaining"], name, self:FormatChatTime(time-elapsed)), "BATTLEGROUND")
		else
			if CappingFrame:IsVisible() then
				CappingFrame:Hide()
			else
				CappingFrame:Show()
			end
		end
	elseif button == "RightButton" then
		if IsShiftKeyDown() then
			self:StopBar(name)
		else
			Capping:Options()
		end
	end
end

function Capping:FormatChatTime(secs)
	return string.format("%d:%02d", secs / 60, math.fmod(secs, 60))
end



